import pandas as pd
import argparse
import matplotlib.pyplot as plt
import numpy as np
from scipy.integrate import quad
from scipy.optimize import curve_fit

def low_pass(f, fl):
    return (1 + (f / fl) ** 4) ** (-0.5)

def high_pass(f, fh):
    return ((f / fh) ** 2) * (1 + (f / fh) ** 4) ** (-0.5)

def effective_bandwidth(lp_func, hp_func):
    integrand = lambda f: (lp_func(f) ** 2) * (hp_func(f) ** 2)
    result, _ = quad(integrand, 0, np.inf)
    return result

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Read a CSV file, filter data, process gain, fit functions, and save outputs.")
    parser.add_argument("file_path", type=str, help="Path to the CSV file")
    parser.add_argument("output_gain_plot", type=str, help="Name of the output file for the gain plot")
    parser.add_argument("output_fit_plot", type=str, help="Name of the output file for the fitted gain plot")
    args = parser.parse_args()
    
    try:
        df = pd.read_csv(args.file_path)
    except Exception as e:
        print(f"Error processing the CSV file: {e}")
        exit()
    
    df.columns = df.columns.str.strip()
    df = df[df["For_Analysis"] == True]
    df.set_index("Frequency [Hz]", inplace=True)
    
    # Compute Gain G(f) = Ch2 / Ch1
    df["Gain"] = df["Ch2 RMS [V]"] / df["Ch1 RMS [V]"]
    
    # Plot Gain G(f) vs Frequency (Logarithmic X-Axis)
    plt.figure(figsize=(10, 6))
    plt.semilogx(df.index, df["Gain"], 'ko-', label="Measured Gain")
    plt.xlabel("Frequency [Hz]")
    plt.ylabel("Gain G(f)")
    plt.title("Gain vs Frequency")
    plt.legend()
    plt.grid(True, which="both", linestyle="--", linewidth=0.5)
    plt.savefig(args.output_gain_plot)
    print(f"Gain plot saved as {args.output_gain_plot}")
    
    # Fit Low Pass and High Pass Functions
    freqs = df.index.to_numpy()
    gains = df["Gain"].to_numpy()
    popt_lp, _ = curve_fit(low_pass, freqs, gains, p0=[1000])  # Initial guess for fl
    popt_hp, _ = curve_fit(high_pass, freqs, gains, p0=[1000])  # Initial guess for fh
    
    # Generate fitted curves
    freqs_fit = np.logspace(np.log10(min(freqs)), np.log10(max(freqs)), 1000)
    gain_lp_fit = low_pass(freqs_fit, *popt_lp)
    gain_hp_fit = high_pass(freqs_fit, *popt_hp)
    
    # Plot fitted curves
    plt.figure(figsize=(10, 6))
    plt.semilogx(freqs, gains, 'ko', label="Measured Gain")
    plt.semilogx(freqs_fit, gain_lp_fit, 'r-', label=f"Low-Pass Fit (fl={popt_lp[0]:.2f} Hz)")
    plt.semilogx(freqs_fit, gain_hp_fit, 'b-', label=f"High-Pass Fit (fh={popt_hp[0]:.2f} Hz)")
    plt.xlabel("Frequency [Hz]")
    plt.ylabel("Gain G(f)")
    plt.title("Gain vs Frequency with Fit")
    plt.legend()
    plt.grid(True, which="both", linestyle="--", linewidth=0.5)
    plt.savefig(args.output_fit_plot)
    print(f"Fitted Gain plot saved as {args.output_fit_plot}")
    
    # Compute Effective Bandwidth
    delta_f_eff = effective_bandwidth(lambda f: low_pass(f, *popt_lp), lambda f: high_pass(f, *popt_hp))
    df["Effective Bandwidth"] = delta_f_eff
    print(df)
