import pandas as pd
import argparse
import matplotlib.pyplot as plt
import numpy as np

def plot_v_meter_squared_vs_rin(R_in, V_meter_squared, fit_line, VJ_squared, VN_squared, output_file):
    """Plot V_meter^2 vs R_in with the fit line."""
    plt.figure(figsize=(8, 6))
    plt.scatter(R_in, V_meter_squared, label="Measured Data", color="blue")
    plt.plot(R_in, fit_line, label=f"Fit: $V_J^2={VJ_squared:.4f}$, $V_N^2={VN_squared:.4f}$", color="red")
    plt.xlabel("Resistance $R_{in}$ [$\\Omega$]")
    plt.ylabel("$V_{meter}^2$ [V²]")
    plt.legend()
    plt.grid()
    plt.title("Measured Noise vs. Resistance with Fit")
    plt.tight_layout()
    plt.savefig(output_file)
    plt.show()

def plot_residuals_vs_rin(R_in, residuals, output_file):
    """Plot residuals of the fit."""
    plt.figure(figsize=(8, 6))
    plt.scatter(R_in, residuals, color="black")
    plt.axhline(0, color='red', linestyle='--')
    plt.xlabel("Resistance $R_{in}$ [$\\Omega$]")
    plt.ylabel("Residuals")
    plt.grid()
    plt.title("Residuals of the Fit")
    plt.tight_layout()
    plt.savefig(output_file)
    plt.show()

def plot_johnson_noise_contribution(R_in, Johnson_noise_contrib, output_file):
    """Plot Johnson noise contribution vs R_in."""
    plt.figure(figsize=(8, 6))
    plt.scatter(R_in, Johnson_noise_contrib, label="Johnson Noise Contribution", color="green")
    plt.xlabel("Resistance $R_{in}$ [$\\Omega$]")
    plt.ylabel("$V_J^2$ [V²]")
    plt.legend()
    plt.grid()
    plt.title("Johnson Noise Contribution vs. Resistance")
    plt.tight_layout()
    plt.savefig(output_file)
    plt.show()

def main():
    # Set up command-line argument parsing
    parser = argparse.ArgumentParser(description="Analyze Johnson noise data")
    parser.add_argument("csv_file", type=str, help="Path to input CSV file")
    parser.add_argument("output_files", nargs=3, type=str, help="Paths to save output plots (3 plots)")
    args = parser.parse_args()

    # Load the CSV file
    df = pd.read_csv(args.csv_file)

    # Strip spaces from column names to avoid errors
    df.columns = df.columns.str.strip()

    # Extracting correct columns
    R_in = df["R_in [Ohm]"]  # Resistance values
    V_meter = df["V_meter [V]"]  # Measured voltage

    # Compute squared values
    V_meter_squared = V_meter**2

    # Perform linear fit: V_meter^2 = a * R_in + b
    coeffs = np.polyfit(R_in, V_meter_squared, 1)
    VJ_squared = coeffs[0]  # Slope (Johnson noise contribution)
    VN_squared = coeffs[1]  # Intercept (Amplifier noise)

    # Generate fitted values
    fit_line = VJ_squared * R_in + VN_squared

    # Compute residuals
    residuals = V_meter_squared - fit_line

    # Compute only Johnson noise contribution (without amplifier noise)
    Johnson_noise_contrib = VJ_squared * R_in

    # Plot and save each of the three plots to different output files
    plot_v_meter_squared_vs_rin(R_in, V_meter_squared, fit_line, VJ_squared, VN_squared, args.output_files[0])
    plot_residuals_vs_rin(R_in, residuals, args.output_files[1])
    plot_johnson_noise_contribution(R_in, Johnson_noise_contrib, args.output_files[2])

    # Print results
    print(f"Fit Results: V_J^2 (slope) = {VJ_squared:.4f}, V_N^2 (intercept) = {VN_squared:.4f}")

if __name__ == "__main__":
    main()

