import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import argparse
from scipy.optimize import curve_fit

# Step 1: Set up argparse to get CSV file and output path from command line
parser = argparse.ArgumentParser(description="Analyze Johnson Noise and Amplifier Noise from CSV data.")
parser.add_argument("csv_file", type=str, help="Path to the CSV file containing the data.")
parser.add_argument("output_file", type=str, help="Path to save the output plot (e.g., output.png).")
args = parser.parse_args()

# Step 2: Load the data
df = pd.read_csv(args.csv_file)
df.columns = df.columns.str.strip()

# Step 3: Extract relevant columns (modify column names based on actual CSV structure)
R_in = df["R_in [Ohm]"]  # Resistance values
G_2 = df["G_2 [1]"]  # Gain values
V_meter = df["V_meter [V]"]  # Measured voltages

# Step 4: Define function for fitting
def noise_model(R, V_N):
    V_J2 = (600 * G_2 / 10) ** 2  # Johnson noise squared
    return np.sqrt(V_J2 + V_N**2)

# Step 5: Fit the data
V_meter_squared = V_meter ** 2
popt, pcov = curve_fit(noise_model, R_in, V_meter_squared)
V_N_fitted = popt[0]

# Step 6: Plot results
plt.figure(figsize=(8, 5))
plt.scatter(R_in, V_meter_squared, label="Measured Data", color="blue")
plt.plot(R_in, noise_model(R_in, V_N_fitted), label=f"Fitted Curve (V_N={V_N_fitted:.3f})", color="red")
plt.xlabel("Resistance (Ohms)")
plt.ylabel("$V_{meter}^2$ (V²)")
plt.legend()
plt.title("Decomposition of Measured Noise")

# Step 7: Save plot to specified output file
plt.savefig(args.output_file, dpi=300)
print(f"Plot saved as {args.output_file}")
