import pandas as pd
import argparse
import numpy as np
import matplotlib.pyplot as plt

# Constants
k_B = 1.38e-23  # Boltzmann constant (J/K)
T = 300  # Room temperature in Kelvin

def process_frequency_dependence(csv_file, output_file):
    """
    Analyze the dependence of Johnson noise on bandwidth.
    Reads data from csv_file, calculates bandwidth, and generates a log-log plot.
    Saves the plot to output_file.
    """
    # Load the CSV file
    df = pd.read_csv(csv_file)
    
    # Strip spaces from column names to avoid issues
    df.columns = df.columns.str.strip()
    
    # Extract relevant data
    R_fixed = df["R [Ohm]"].unique()[0]
    f_high = df["f_high [Hz]"]  # High-pass cutoff frequencies
    f_low = df["f_low [Hz]"]  # Low-pass cutoff frequencies
    V_meter = df["V_meter [V]"]  # Measured voltage values

    # Compute squared values (voltage noise power)
    V_meter_squared = V_meter ** 2

    # Compute bandwidth (Δf)
    bandwidth = f_high - f_low

    # Compute theoretical Johnson noise
    VJ_theoretical_squared = 4 * k_B * T * R_fixed * bandwidth

    # Create a log-log plot
    plt.figure(figsize=(8, 6))
    plt.scatter(bandwidth, V_meter_squared, color="blue", label="Measured Data")
    plt.plot(bandwidth, VJ_theoretical_squared, color="red", linestyle="--", label="Theoretical Johnson Noise")
    plt.xscale("log")
    plt.yscale("log")
    plt.xlabel("Bandwidth Δf [Hz]")
    plt.ylabel("$V_{meter}^2$ [V²]")
    plt.title(f"Johnson Noise vs. Bandwidth")
    plt.grid(True, which="both", linestyle="--", linewidth=0.5)
    plt.legend()

    # Save and show the plot
    plt.savefig(output_file)
    plt.show()

if __name__ == "__main__":
    # Set up command-line arguments
    parser = argparse.ArgumentParser(description="Analyze frequency dependence of Johnson noise")
    parser.add_argument("csv_file", type=str, help="Path to input CSV file")
    parser.add_argument("output_file", type=str, help="Path to save output plot")
    args = parser.parse_args()

    # Run the analysis
    process_frequency_dependence(args.csv_file, args.output_file)

